@@ -1,9 +1,12 @@
 import mimetypes
+import os
+from os.path import normpath
+from pathlib import Path
 
 import chevron
 import markdown
+from markdownup.access_control import AccessControl
 from markdownup.markdown_directory import MarkdownDirectory
-from markdownup.path_resolver import resolve_str
 from markdownup.theme import Theme
 
 
@@ -11,7 +14,9 @@ class MarkdownUp:
 
     def __init__(self, config):
         self.config = config
-        self.root = MarkdownDirectory(config)
+        self.access_control = AccessControl(config)
+        self.root_path = Path(config['content']['root']).resolve()
+        self.root = MarkdownDirectory(self)
         self.theme = Theme(config)
 
     def wsgi_app(self, environ, start_response):
@@ -23,20 +28,34 @@ class MarkdownUp:
             return ['405 Method Not Allowed']
 
         request_path = environ['PATH_INFO'] or '/'
-        request_path = request_path[1:]
 
         response = self.get(request_path)
         start_response(response.status, response.headers)
         yield from response.body
 
-    def get(self, path: str) -> 'Response':
+    def get(self, path: str, environ=None) -> 'Response':
 
-        markdown_file = self.root.resolve(path)
+        environ = environ or {}
 
+        # normalize path
+        if not path.startswith('/'):
+            return Response('400 Bad Request')
+        abs_path = normpath(path)
+        abs_path = Path(abs_path)
+        rel_path = Path(*abs_path.parts[1:])
+
+        # serve markdown
+        markdown_file = self.root.resolve(rel_path)
         if markdown_file:
 
+            if not self.access_control.is_access_allowed(abs_path):  # TODO use environ for more advanced access control
+                return Response('403 Forbidden')
+
             source = markdown_file.read()
 
+            # relying on the fact that not process handles multiple requests at the same time, we can safely change wd
+            os.chdir(markdown_file.path.parent)
+
             html = chevron.render(
                 template=self.theme.frame,
                 partials_path=str(self.theme.path),
@@ -55,13 +74,24 @@ class MarkdownUp:
                 (bytes(b, 'UTF-8') for b in html.splitlines(keepends=True))
             )
 
-        try:
-            path = resolve_str(self.root.path, path) or \
-                   resolve_str(self.theme.path, path)
-            if not path.is_file():
-                return Response('404 Not Found')
-        except ValueError:
-            return Response('400 Bad Request')
+        # serve asset file
+        asset_file = self.root_path / rel_path
+        if asset_file.is_file():
+            if not self.access_control.is_access_allowed(abs_path):  # TODO use environ for more advanced access control
+                return Response('403 Forbidden')
+            else:
+                return self.serve_file(asset_file)
+
+        # serve theme file
+        theme_file = self.theme.path / rel_path
+        if theme_file.is_file():
+            return self.serve_file(theme_file)
+
+        # serve 404
+        return Response('404 Not Found')
+
+    @staticmethod
+    def serve_file(path: Path):
 
         guessed_mimetype = mimetypes.guess_type(path.name)[0]
 
@@ -84,4 +114,4 @@ class Response:
     def __init__(self, status: str, headers=None, body=None):
         self.status = status
         self.headers = headers or []
-        self.body = body or iter(())
+        self.body = body or iter([bytes(status, 'UTF-8')])
